
# Provide some help and options!
#
# Example:
#   >> options = option_parser(ARGV)
#   => puts options[:option]
#
# Arguments:
#   ARGV: (Array or command line parameters)
def option_parser(opts)
  options = {}
  OptionParser.new do |opts|
    opts.banner = "Usage: phoseum-cli [options]\n\nRead about details on album names and types of images supported on official website:\nphoseum.org\n\n".green

    opts.on("-a", "--album-path STRING", "Choose destination album. Use with: [update, delete, check]") do |a|
      options[:album] = a
    end

    opts.on("-c", "--check", "Check against Phoseum API Server. Use with: [album-path, image, <alone>]") do |c|
      options[:check] = c
    end

    opts.on("-C", "--create-user USERNAME", "Create user on the application") do |cre|
      options[:create_user] = cre
    end

    opts.on("-d", "--delete", "Delete Album or Image. Use with: [album-path, image]") do |d|
      options[:delete] = d
    end

    opts.on("-D", "--description 'SHORTDESC'", "Add album description (up to 255 chars). Use with: [album-path]") do |ds|
      options[:description] = ds
    end

    opts.on("-f", "--file-cfg FILENAME", "Use alternative configuration file") do |f|
      options[:filecfg] = f
    end

    opts.on("-i", "--image FILENAME", "Manipulate one image. Use with: [check, upload, delete]") do |i|
      options[:image] = i
    end

    opts.on("-l", "--login", "Make a login to save a new token") do |l|
      options[:login] = l
    end

    opts.on("-m", "--multiple", "Ignore repeated images, keep multiple") do |m|
      options[:multiple] = m
    end

    opts.on("-N", "--name 'REALNAME'", "Set Album's synthetic name. Use with: [album-path]") do |n|
      options[:name] = n
    end

    opts.on("-v", "--verbose", "Run verbosely (DEBUG mode)") do |v|
      options[:verbose] = v
    end

    opts.on("-V", "--version", "What version we are") do |vs|
      options[:version] = vs
    end

    opts.on("-x", "--cross-versions", "Check version match between Client and API") do |x|
      options[:crossv] = x
    end

    opts.on("-r", "--role ROLE", "Role for user: [User, Super]. Use with 'create-user'") do |r|
      options[:role] = r
    end

    opts.on("-o", "--options", "Update options from User. Use with: [user]") do |o|
      options[:options] = o
    end

    opts.on("-q", "--quiet", "Avoid regular info, useful for JSON output") do |q|
      options[:quiet] = q
    end

    opts.on("-t", "--test", "Test local files. Used to identify on local directory what will be uploaded.") do |t|
      options[:test] = t
    end

    opts.on("-u", "--upload", "Upload files to Phoseum API Server. Use with: [album-path, image]") do |u|
      options[:upload] = u
    end

    opts.on("-U", "--User USERNAME", "Username for delete or change password / role. Use with: [delete, options]") do |usr|
      options[:user] = usr
    end

    opts.on("-h", "-h", "That is this...") do |h|
      options[:help] = h
    end

  end.parse!
  return options
end

# Give yes/no prompt!
#
# Example:
#   >> confirm_action("message")
#   => You must write 'YES' to confirm, otherwise NO is assumed
#   => message :[YES/NO]:
#
# Arguments:
#   msg: (String)
def confirm_action(msg)
  puts "You must write 'YES' to confirm, otherwise NO is assumed".yellow
  print "#{msg} :[YES/NO]: "
  confirm=STDIN.gets.chomp
  if confirm == "YES"
    return true
  else
    return false
  end
end

class String
  # colorization
  def colorize(color_code)
    "\e[#{color_code}m#{self}\e[0m"
  end

  def red
    colorize(31)
  end

  def green
    colorize(32)
  end

  def yellow
    colorize(33)
  end

  def blue
    colorize(34)
  end

  def pink
    colorize(35)
  end

  def light_blue
    colorize(36)
  end
end

def client_checks
  if $config['SERVERURL'] == ''
    puts "I could not find a valid SERVERURL configuration. CFG is empty.".red
    exit 1
  elsif $config['SERVERURL'] !~ /\A#{URI::regexp(['https', 'http'])}\z/
    puts "I could not find a valid SERVERURL configuration. Contains: #{$config['SERVERURL']}".red
    exit 1
  end
  if !$config['SERVERURL']
    puts "I could not find the SERVERURL from Phoseum config, this client is then useless.".red
    exit 1
  end
  if !$config['PORT']
    puts "I could not find the PORT from Phoseum config, this client is then useless.".red
    exit 1
  end
end

def clean_html(msg)
  msg.gsub!(/<\/?[^>]*>/, ' ')
  msg.gsub!(/\n\n+/, '\n')
  msg.gsub!(/^\n|\n$/, ' ')
  return msg
end

def clean_name(nameval)
  name_words = nameval.split(' ')
  name = ''
  name_words.each do |word|
    cword = check_string_sanity(word)
    name = name.nil? ? cword : [name,cword].join(' ')
  end
  name.gsub!(/^\ /,'')
  return name
end

def check_image_exists(fpath)
  flocal = fpath.split('/')
  filename = flocal.last
  album = fpath.clone
  album.gsub!(/#{filename}/,'')

  if ffile = search_image(filename,album)
    return ffile
  else
    return false
  end
  exit 0
end

def search_image(sign,album='')
  if !validate_token($config['TOKEN'])
    if token = user_login()
      headers = { "bearer" => token }
    else
      puts "Login Failed".red
      exit 1
    end
  else
    headers={ "bearer" => "#{$config['TOKEN']}" }
  end
  some_uri= album == '' ? "?signature=#{sign}" : "?signature=#{sign}&inside_album=#{album}"
  base = URI.parse("#{$config['SERVERURL']}#{some_uri}")
  request = Net::HTTP::Get.new(base,headers)
  response = Net::HTTP.start(base.hostname, $config['PORT'],
                             :timeout => $config['CALL_TIMEOUT'],
                             :use_ssl => base.scheme == "https",
                             :verify_mode => OpenSSL::SSL::VERIFY_PEER,
                             :ca_file => $config['CA_TRUST']
                             ) do |http|
    http.request(request)
  end
  begin
    list = JSON.parse(response.body)
    if list['warning']
      # puts list['warning']
      return list['warning']
    end
    if list['success']
      return false
    end
  rescue
    puts "\nThe server sent out an Error:".red
    puts clean_html(response.body)
    exit 1
  end
end

def api_caller(json_body,auth=false,cli=false)
  headers = {}
  if !auth
    if $config['TOKEN']
      headers={ "bearer" => "#{$config['TOKEN']}" }
    else
      return false
    end
  end
  base = URI.parse("#{$config['SERVERURL']}")
  request = Net::HTTP::Post.new(base,headers)
  request.body = JSON.generate(json_body)

  if auth
    if cli
      request.basic_auth("cli", "loginNOauth")
    else
      request.basic_auth("auth", $config['DEFAULT_SECRET'])
    end
  else
    if cli
      request.basic_auth("cli", "loginNOauth")
    end
  end

  response = Net::HTTP.start(base.hostname, $config['PORT'],
                             :timeout => $config['CALL_TIMEOUT'],
                             :use_ssl => base.scheme == "https",
                             :verify_mode => OpenSSL::SSL::VERIFY_PEER,
                             :ca_file => $config['CA_TRUST']
                             ) do |http|
    http.request(request)
  end
  begin
    list = JSON.parse(response.body)
    return list
  rescue
    if !auth
      puts "\nThe server sent out an Error:".red
      puts clean_html(response.body)
      exit 1
    else
      return '{"error": "Secret is invalid"}'
    end
  end
end

def validate_token(token)
  result = api_caller({"action" => "check-token" })
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end

def change_user_password(user,pass,auth=false)
  result = api_caller({"action" => "change-password", "username" => user, "password" => pass},auth)
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end

def change_username(user,new_user)
  result = api_caller({"action" => "change-username", "username" => user, "new-username" => new_user},true)
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end

def add_user(user,password,role)
  result = api_caller({"action" => "create-user", "user" => user, "password" => password, "role" => role},true)
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end


def change_role(user,new_role)
  result = api_caller({"action" => "change-role", "username" => user, "new-role" => new_role},true)
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end

def password_sanity(password)
  if password.length < MIN_PASS
    puts "New password is shorter than #{MIN_PASS} chars, please use a bigger password.".red
    exit 1
  end
  if !password.test_password
    puts "Password must be at least #{MIN_PASS} and contain at least one capital, one symbol, one number, one regular characters.".red
    exit 1
  end
end

def username_sanity(username)
  if username.length < MIN_USER
    puts "New Username is shorter than #{MIN_USER} chars, please use a bigger username.".red
    exit 1
  end
end

def cross_versions
  return api_caller({"action" => "version-check" },true,true)
end

def delete_user(username)
  result = api_caller({"action" => "delete-user", "username" => username},true)
  if result['error']
    return false
  end
  if result['success']
    return true
  end
end

def local_version
  puts "This CLI library is running version: #{VERSION_SIGN}"
  return
end

VERSION_SIGN="0.0.26"
